local sti = require "sti"
local lume = require "lume"

function love.load()
  map = sti.new("tilemaps/testmap.lua")
  
  love.graphics.setDefaultFilter( 'nearest', 'nearest' )
  
  local layer = map:addCustomLayer("Sprites", 7)

  local player
  for k, object in pairs(map.objects) do
    if object.name == "player" then
      player = object
      break
    end
  end

  local sprite = love.graphics.newImage("assets/player.png")
    layer.player = {
      sprite = sprite,
      x = player.x + 8,
      y = player.y + 16,
      ox = sprite:getWidth()/2, -- no offset required by current sprite
      oy = sprite:getHeight()
    }
    
  layer.update = function(self, dt)
    local speed = 50
    mouseDown = love.mouse.isDown(1)
    mouseDist = math.floor(lume.distance(self.player.x, self.player.y, mX, mY)) -- find distance from character and mouse when holding left click
    angleM = math.deg(lume.angle(self.player.x, self.player.y, mX, mY))
    timeM = mouseDist/speed --* dt -- finds the time in seconds
    slope, xDif, yDif = slopeFinder(self.player.x, self.player.y, mX, mY)
    if mouseDown then -- mouse movement.
      xMod, yMod = xDif/timeM, yDif/timeM
      --print(xDif.." "..xMod.." "..yDif.." "..yMod)
      print(slope)
      if lume.sign(slope) == 1 then -- positive
        if xDif or yDif ~= 0 or inf then
          self.player.x = self.player.x - (xMod) * dt
          self.player.y = self.player.y - (yMod) * dt
        end
      elseif lume.sign(slope) == -1 then -- negative
        if xDif or yDif ~= 0 or inf then
          self.player.x = self.player.x + (xMod) * dt
          self.player.y = self.player.y + (yMod) * dt
        end
      end
    end
    if love.keyboard.isDown("w") or love.keyboard.isDown("up") then -- all keyboard movement from here down in this function.
      if love.keyboard.isDown("a") or love.keyboard.isDown("left") then
        self.player.y = self.player.y - speed * dt
        self.player.x = self.player.x - speed * dt
      elseif love.keyboard.isDown("d") or love.keyboard.isDown("right") then
        self.player.y = self.player.y - speed * dt
        self.player.x = self.player.x + speed * dt
      else
        self.player.y = self.player.y - speed * dt
      end
    elseif love.keyboard.isDown("s") or love.keyboard.isDown("down") then
      if love.keyboard.isDown("a") or love.keyboard.isDown("left") then
        self.player.y = self.player.y + speed * dt
        self.player.x = self.player.x - speed * dt
      elseif love.keyboard.isDown("d") or love.keyboard.isDown("right") then
        self.player.y = self.player.y + speed * dt
        self.player.x = self.player.x + speed * dt
      else
        self.player.y = self.player.y + speed * dt
      end
    elseif love.keyboard.isDown("a") or love.keyboard.isDown("left") then
      self.player.x = self.player.x - speed * dt
    elseif love.keyboard.isDown("d") or love.keyboard.isDown("right") then
      self.player.x = self.player.x + speed * dt
    end
  end

  layer.draw = function(self)
    love.graphics.draw(
      self.player.sprite,
      math.floor(self.player.x),
      math.floor(self.player.y),
      0,
      1,
      1,
      self.player.ox,
      self.player.oy
    )

    love.graphics.setPointSize(5)
    --love.graphics.points(math.floor(self.player.x), math.floor(self.player.y))
  end

  -- Remove unneeded object layer
  map:removeLayer("Spawn Point")
end

function slopeFinder(x1, y1, x2, y2)
  local xDif, yDif = (x1 - x2), (y1 - y2) -- ensure that x1/y1 are the player!
  local slope = yDif/xDif
  return slope, yDif, xDif
end

function love.update(dt)
  mX, mY = love.mouse.getPosition()
  map:update(dt)
end

function love.draw()
  map:draw()
end